/*---------------------------------------------------------------------------*/

/*!  \file  bfs.c
     \brief  breadth first search library.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005,2006 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include "xmalloc.h"
#include "bfs.h"

/*---------------------------------------------------------------------------*/
typedef struct queue_item
{
    int item;
    struct queue_item *next;
} queue_item;

typedef struct queue
{
    queue_item *head, *tail;
} queue;

/*---------------------------------------------------------------------------*/
static queue *allocate_queue(void);
static void free_queue(queue *q);
static void push_queue(queue *q, int item);
static int pop_queue(queue *q);
static int queue_is_empty(queue *q);

/*---------------------------------------------------------------------------*/
void breadth_first_search(int n, int **adjacent,
                          double **distance, double *weight)
{
    int i, j;
    int start;
    queue *q;

    for(start = 0;start < n;start++)
    {
        q = allocate_queue();

        for(i = 0;i < n; i++)
            distance[start][i] = -1;
        push_queue(q, start);
        distance[start][start] = 0;


        i = pop_queue(q);
        for(j = 0;j < n;j++)
        {
            if(adjacent[i][j] && (distance[start][j] < 0))
            {
                push_queue(q, j);
                distance[start][j] = 0;
            }
        }

        while(!queue_is_empty(q))
        {
            i = pop_queue(q);
            for(j = 0;j < n;j++)
            {
                if(adjacent[i][j] && (distance[start][j] < 0))
                {
                    push_queue(q, j);
                    distance[start][j] = distance[start][i] + weight[i];
                }
            }
        }


        free_queue(q);
    }
}


/*---------------------------------------------------------------------------*/
queue *allocate_queue(void)
{
    queue *q;

    q = xmalloc(sizeof(queue));
    q->head = q->tail = xmalloc(sizeof(queue_item));

    return q;
}

/*---------------------------------------------------------------------------*/
void free_queue(queue *q)
{
    while(q->head != q->tail)
        pop_queue(q);
    free(q->head);
    free(q);
}

/*---------------------------------------------------------------------------*/
void push_queue(queue *q, int item)
{
    q->tail->item = item;

    q->tail->next = xmalloc(sizeof(queue_item));
    q->tail->next->next = NULL;
    q->tail->next->item = -1;

    q->tail = q->tail->next;
}

/*---------------------------------------------------------------------------*/
int pop_queue(queue *q)
{
    int item;
    queue_item *head;

    if(q->head == q->tail)
        return 0;

    head = q->head;
    q->head = q->head->next;

    item = head->item;
    free(head);

    return item;
}

/*---------------------------------------------------------------------------*/
int queue_is_empty(queue *q)
{
    return (q->head == q->tail);
}

/*---------------------------------------------------------------------------*/
