/*---------------------------------------------------------------------------*/

/*!  \file  correlation.c
     \brief  calculate the correlation function for Gaussian chains.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005,2006 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_MATH_H
#include <math.h>
#endif

#include "matrix.h"
#include "correlation.h"

/*---------------------------------------------------------------------------*/
double h1(polymer_chain *chain,int i,double q2)
{
    double N,f;
    double x,h;

    N = chain->N;
    f = chain->f[i];
    x = (1.0 / 6.0) * N * f * (chain->b[i] * chain->b[i]) * q2;

        /*  the self correlation function of a subchain.  */
    h = N * (f * f) * 2.0 / (x * x) * (exp(-x) - 1.0 + x);

    return h;
}

/*---------------------------------------------------------------------------*/
double h2(polymer_chain *chain,int i,int j,double q2)
{
    double N,fi,fj,l2;
    double xi,xj,h;

    N = chain->N;
    fi = chain->f[i];
    fj = chain->f[j];
    xi = (1.0 / 6.0) * N * fi * (chain->b[i] * chain->b[i]) * q2;
    xj = (1.0 / 6.0) * N * fj * (chain->b[j] * chain->b[j]) * q2;
    l2 = chain->l2[i][j];

        /*  the correlation function between two subchains.  */
    h = N * fi * fj / (xi * xj) * (exp(-xi) - 1.0) * (exp(-xj) - 1.0)
        * exp(-(1.0 / 6.0) * l2 * q2);

    return h;
}

/*---------------------------------------------------------------------------*/
void calculate_H01(polymer_chain *chain,double **H01)
{
    int i,j,ns;
    double N,*f,*b,**l2;

    ns = chain->ns;
    N = chain->N;
    f = chain->f;
    b = chain->b;
    l2 = chain->l2;

    for(i = 0;i < ns;i++)
    {
        for(j = 0;j < i;j++)
        {
            H01[i][j] = N * f[i] * f[j]
                * (((1.0 / 12.0) * N * f[i] * (b[i] * b[i]))
                   + ((1.0 / 12.0) * N * f[j] * (b[j] * b[j]))
                   + ((1.0 / 6.0) * l2[i][j]));
            H01[j][i] = H01[i][j];
        }

        H01[i][i] = (1.0 / 18.0) * (N * N) * (f[i] * f[i] * f[i])
            * (b[i] * b[i]);
    }
}

/*---------------------------------------------------------------------------*/
void calculate_H02(polymer_chain *chain,double **H02)
{
    int i,j,ns;
    double N,*f,*b,**l2;

    ns = chain->ns;
    N = chain->N;
    f = chain->f;
    b = chain->b;
    l2 = chain->l2;

    for(i = 0;i < ns;i++)
    {
        for(j = 0;j < i;j++)
        {
            H02[i][j] = N * f[i] * f[j]
                * (((1.0 / 216.0) * (N * N) * (f[i] * f[i])
                    * (b[i] * b[i] * b[i] * b[i]))
                   + ((1.0 / 216.0) * (N * N) * (f[j] * f[j])
                      * (b[j] * b[j] * b[j] * b[j]))
                   + ((1.0 / 144.0) * (N * N) * f[i] * f[j]
                      * (b[i] * b[i]) * (b[j] * b[j]))
                   - ((1.0 / 72.0) * N * f[i] * (b[i] * b[i]) * l2[i][j])
                   - ((1.0 / 72.0) * N * f[j] * (b[j] * b[j]) * l2[i][j])
                   + ((1.0 / 72.0) * (l2[i][j] * l2[i][j])));
            H02[j][i] = H02[i][j];
        }

        H02[i][i] = (1.0 / 432.0) * (N * N * N) * (f[i] * f[i] * f[i] * f[i])
            * (b[i] * b[i] * b[i] * b[i]);
    }
}

/*---------------------------------------------------------------------------*/
void calculate_Hi2(polymer_chain *chain,double **Hi2)
{
    int i,j,ns;
    double N,*b,**l2;

    ns = chain->ns;
    N = chain->N;
    b = chain->b;
    l2 = chain->l2;

    for(i = 0;i < ns;i++)
    {
        for(j = 0;j < i;j++)
        {
            if(l2[i][j] == 0.0)
            {
                Hi2[i][j] = 36.0 / (N * (b[i] * b[i]) * (b[j] * b[j]));
            }
            else
            {
                Hi2[i][j] = 0;
            }
            Hi2[j][i] = Hi2[i][j];
        }

        Hi2[i][i] = -72.0 / (N * (b[i] * b[i] * b[i] * b[i]));
    }
}

/*---------------------------------------------------------------------------*/
