/*---------------------------------------------------------------------------*/

/*!  \file  drops.c
     \brief  density functional simulator for block copolymers  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005-2014 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS 0
#endif
#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1
#endif

#include "getopt.h"
#include "chemicalpotential.h"
#include "condition.h"
#include "simulator.h"
#include "mt.h"
#include "textread.h"
#include "externalforce.h"
#include "iofiles.h"
#include "luaread.h"
#include "mgpoisson.h"

/*---------------------------------------------------------------------------*/
static char *programstring = "drops";

static struct option longopts[] =
{
    {"input", required_argument, NULL, 'i'},
    {"psi", required_argument, NULL, 'p'},
    {"external", required_argument, NULL, 'e'},
    {"version", no_argument, NULL, 'v'},
    {"help", no_argument, NULL, 'h'},
    {0, 0, NULL, 0}
};

static char *optstring = "i:p:e:vh";

static char *helpstring =
"\
Usage: %s [options] [input]\n\
Options:\n\
  -i, --input <file>       input file for the simulator\n\
  -p, --psi <file>         psi-field file for the initial condition\n\
  -e, --external <file>    external force field file for the evolver\n\
  -v, --version            output version information and exit\n\
  -h, --help               display this help and exit\n\
";

static char *copyingstring =
"\
%s (%s) %s\n\
\n\
Copyright (C) 2005-2014 Takashi Uneyama\n\
This is free software; see the source for copying conditions.  There is NO\n\
warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
";

/*---------------------------------------------------------------------------*/
static int process_option(int argc,char **argv,char **input_lua,char **density_input,
                          char **external_input);

/*---------------------------------------------------------------------------*/
int main(int argc,char **argv)
{
    char *input_lua = DEFAULT_INPUT_FILE;
    char *density_input = NULL;
    char *external_input = NULL;
    field_geometry *geometry;
    simulation_condition *condition;
    simulation_iofiles *iofiles;
    polymer_blend *blend;
    density_field *density;
    field_fftw_plan *plan;
    chemical_potential *potential;
    external_force *force;
    multigrid_condition *mgcondition;

    if(process_option(argc,argv,&input_lua,&density_input,&external_input) != 0)
    {
        return EXIT_FAILURE;
    }

        /*  read the condition.  */
    read_input_lua(input_lua,&condition,&iofiles,&geometry,&blend,&mgcondition);
    allocate_q2_q2i(geometry);

    mtinit(condition->seed);

    calculate_blend_propagator(blend);

    density = allocate_density_field(blend,geometry);
    potential = allocate_chemical_potential(blend,geometry);
    plan = allocate_field_fftw_plan(geometry,iofiles->wisdom);


        /*  initialize the density field.  */
    set_density_field_random(density,condition->initial_noise);
    if(density_input != NULL)
    {
        read_density_field_text(density_input,density);
        reset_density_field_offset(density);
    }

        /*  set the external force field.  */
    force = NULL;
    if(external_input != NULL)
    {
        force = allocate_external_force(blend,geometry);
        read_external_force_text(external_input,force);
    }


        /*  simulation.  */
    simulate(geometry,condition,iofiles,blend,density,force,plan,potential,mgcondition);


    free_q2_q2i(geometry);
    free_field_fftw_plan(plan);
    free_density_field(density);
    free_chemical_potential(potential);
    free_external_force(force);
    free_field_geometry(geometry);
    free_simulation_condition(condition);
    free_polymer_blend(blend);
    free_multigrid_condition(mgcondition);

    return EXIT_SUCCESS;
}

/*---------------------------------------------------------------------------*/
int process_option(int argc,char **argv,char **input_lua,char **density_input,
                   char **external_input)
{
    int c;
    int option_index;
    char *program_name;

    program_name = argv[0];

    do
    {
        option_index = 0;
        c = getopt_long(argc,argv,optstring,longopts,&option_index);

        switch(c)
        {
            case 'i':
                    /*  input file  */
                *input_lua = optarg;
                break;

            case 'p':
                    /*  psi-field input  */
                *density_input = optarg;
                break;

            case 'e':
                    /*  external force input  */
                *external_input = optarg;
                break;

            case 'v':
                    /*  version  */
                fprintf(stderr,copyingstring,programstring,PACKAGE,VERSION);
                exit(EXIT_SUCCESS);
                break;

            case 'h':
                    /*  help  */
                fprintf(stderr,helpstring,program_name);
                exit(EXIT_SUCCESS);
                break;

            case '?':
                    /*  invalid option  */
                exit(EXIT_FAILURE);
                break;

            case -1:
                break;

            default:
                fprintf(stderr,
                        "error: getopt returned character code 0%o\n", c);
                return 1;
        }
    }
    while(c != -1);


    if(optind < argc)
    {
            /*  input file  */
        *input_lua = argv[optind++];
    }

    return 0;
}

/*---------------------------------------------------------------------------*/
