/*---------------------------------------------------------------------------*/

/*!  \file  evolver.c
     \brief  a field evolver for the statics.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005,2006 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#ifdef HAVE_MATH_H
#include <math.h>
#endif

#ifdef HAVE_FFTW3_H
#ifdef malloc
#undef malloc
#include <fftw3.h>
#define malloc rpl_malloc
#else
#include <fftw3.h>
#endif
#endif

#include "operator.h"
#include "tridiag.h"
#include "evolver.h"
#include "adi.h"

/*---------------------------------------------------------------------------*/
static void evolve_density_adi_1d(density_field *density,
                                  chemical_potential *potential,
                                  external_force *force,
                                  double omega,double noise,
                                  field_fftw_plan *plan,multigrid_condition *mgcondition,
                                  multigrid_field *multigrid,int use_multigrid,
                                  tridiag_matrix *tridiag,double *u);
static void evolve_density_adi_2d(density_field *density,
                                  chemical_potential *potential,
                                  external_force *force,
                                  double omega,double noise,
                                  field_fftw_plan *plan,multigrid_condition *mgcondition,
                                  multigrid_field *multigrid,int use_multigrid,
                                  tridiag_matrix *tridiag,double *u);
static void evolve_density_adi_3d(density_field *density,
                                  chemical_potential *potential,
                                  external_force *force,
                                  double omega,double noise,
                                  field_fftw_plan *plan,multigrid_condition *mgcondition,
                                  multigrid_field *multigrid,int use_multigrid,
                                  tridiag_matrix *tridiag,double *u);
static void set_tridiagonal_matrix(int n,double h,tridiag_matrix *tridiag);
static void calculate_chemical_potential_for_adi(density_field *density,
                                                 chemical_potential *potential,
                                                 external_force *force,
                                                 double omega,double noise,
                                                 field_fftw_plan *plan,
                                                 multigrid_condition *mgcondition,
                                                 multigrid_field *multigrid,
                                                 int use_multigrid);

/*---------------------------------------------------------------------------*/
void evolve_density(density_field *density,
                    chemical_potential *potential,
                    double omega)
{
    int p,i;
    int n;
    int r;
    int np,*ns;
    double ***psi;
    double ***mu;
    double ***mu_ext;

    np = density->blend->np;
    ns = density->blend->ns;
    n = lattice_point_number(density->geometry);

    psi = density->psi;
    mu = potential->mu;
    mu_ext = potential->mu_ext;

    for(p = 0;p < np;p++)
    {
        for(i = 0;i < ns[p];i++)
        {
            for(r = 0;r < n;r++)
            {
                psi[p][i][r] +=
                    - omega * (mu[p][i][r] + mu_ext[p][i][r]);
            }
        }
    }
}

/*---------------------------------------------------------------------------*/
void evolve_density_adi(density_field *density,
                        chemical_potential *potential,
                        external_force *force,
                        double omega,double noise,
                        field_fftw_plan *plan,multigrid_condition *mgcondition,
                        multigrid_field *multigrid,int use_multigrid,
                        tridiag_matrix *tridiag,double *u)
{
    switch(density->geometry->dimension)
    {
        case 1:
            evolve_density_adi_1d(density,potential,force,omega,noise,plan,
                                  mgcondition,multigrid,use_multigrid,tridiag,u);
            break;

        case 2:
            evolve_density_adi_2d(density,potential,force,omega,noise,plan,
                                  mgcondition,multigrid,use_multigrid,tridiag,u);
            break;

        case 3:
            evolve_density_adi_3d(density,potential,force,omega,noise,plan,
                                  mgcondition,multigrid,use_multigrid,tridiag,u);
            break;

        default:
            return;
    }
}

/*---------------------------------------------------------------------------*/
void evolve_density_adi_1d(density_field *density,
                           chemical_potential *potential,
                           external_force *force,
                           double omega,double noise,
                           field_fftw_plan *plan,multigrid_condition *mgcondition,
                           multigrid_field *multigrid,int use_multigrid,
                           tridiag_matrix *tridiag,double *u)
{
    int p,i;
    int nx;
    int np,*ns;
    double ***psi;
    double ***src;
    double ***mu;
    double ***mu_ext;
    double h,b;
    double dx;

    np = density->blend->np;
    ns = density->blend->ns;
    nx = density->geometry->nx;
    dx = density->geometry->lx / density->geometry->nx;

    psi = density->psi;
    src = get_unused_work_array(psi,density);
    mu = potential->mu;
    mu_ext = potential->mu_ext;

    calculate_chemical_potential_for_adi(density,potential,
                                         force,omega,noise,
                                         plan,mgcondition,multigrid,
                                         use_multigrid);


        /*  evolve density field.  */
    for(p = 0;p < np;p++)
    {
        for(i = 0;i < ns[p];i++)
        {
            b = density->blend->chain[p]->b[i];
            b = b * b / 3.0;
            h = dx * dx / omega / b;

            calculate_adi_source_1d(nx,dx * dx,h,omega,
                                    src[p][i],psi[p][i],
                                    mu[p][i],mu_ext[p][i]);

            set_tridiagonal_matrix(nx,h,tridiag);
            solve_cyclic_tridiag(nx,1,1,
                                 tridiag,
                                 src[p][i],u);
        }
    }

    density->psi = src;
    normalize_density_field(density);

    calculate_chemical_potential_incompressible(density->blend,density,potential);
    evolve_multiplier(potential,omega);
    correct_density_field(density);
}

/*---------------------------------------------------------------------------*/
void evolve_density_adi_2d(density_field *density,
                           chemical_potential *potential,
                           external_force *force,
                           double omega,double noise,
                           field_fftw_plan *plan,multigrid_condition *mgcondition,
                           multigrid_field *multigrid,int use_multigrid,
                           tridiag_matrix *tridiag,double *u)
{
    int p,i;
    int nx,ny;
    int np,*ns;
    double ***psi;
    double ***src;
    double ***mu;
    double ***mu_ext;
    double h,b;
    double dx,dy;

    np = density->blend->np;
    ns = density->blend->ns;
    nx = density->geometry->nx;
    ny = density->geometry->ny;
    dx = density->geometry->lx / density->geometry->nx;
    dy = density->geometry->ly / density->geometry->ny;

    psi = density->psi;
    src = get_unused_work_array(psi,density);
    mu = potential->mu;
    mu_ext = potential->mu_ext;



    calculate_chemical_potential_for_adi(density,potential,
                                         force,omega,noise,
                                         plan,mgcondition,multigrid,
                                         use_multigrid);


        /*  evolve density field  for x axis.  */
    for(p = 0;p < np;p++)
    {
        for(i = 0;i < ns[p];i++)
        {
            b = density->blend->chain[p]->b[i];
            b = b * b / 3.0;
            h = 2.0 * dx * dx / omega / b;

            calculate_adi_source_2d_x(nx,ny,dx * dx,dy * dy,h,omega,
                                      src[p][i],psi[p][i],
                                      mu[p][i],mu_ext[p][i]);

            set_tridiagonal_matrix(nx,h,tridiag);
            solve_cyclic_tridiag(nx,ny,ny,
                                 tridiag,
                                 src[p][i],u);
        }
    }
    density->psi = src;

    psi = density->psi;
    src = get_unused_work_array(psi,density);
    normalize_density_field(density);

    calculate_chemical_potential_incompressible(density->blend,density,potential);
    evolve_multiplier(potential,omega);
    correct_density_field(density);


    calculate_chemical_potential_for_adi(density,potential,
                                         force,omega,noise,
                                         plan,mgcondition,multigrid,
                                         use_multigrid);


        /*  evolve density field for y axis.  */
    for(p = 0;p < np;p++)
    {
        for(i = 0;i < ns[p];i++)
        {
            b = density->blend->chain[p]->b[i];
            b = b * b / 3.0;
            h = 2.0 * dy * dy / omega / b;

            calculate_adi_source_2d_y(nx,ny,dx * dx,dy * dy,h,omega,
                                      src[p][i],psi[p][i],
                                      mu[p][i],mu_ext[p][i]);

            set_tridiagonal_matrix(ny,h,tridiag);
            solve_cyclic_tridiag(ny,nx,1,
                                 tridiag,
                                 src[p][i],u);
        }
    }

    density->psi = src;
    normalize_density_field(density);

    calculate_chemical_potential_incompressible(density->blend,density,potential);
    evolve_multiplier(potential,omega);
    correct_density_field(density);
}

/*---------------------------------------------------------------------------*/
void evolve_density_adi_3d(density_field *density,
                           chemical_potential *potential,
                           external_force *force,
                           double omega,double noise,
                           field_fftw_plan *plan,multigrid_condition *mgcondition,
                           multigrid_field *multigrid,int use_multigrid,
                           tridiag_matrix *tridiag,double *u)
{
    int p,i;
    int nx,ny,nz;
    int np,*ns;
    double ***psi;
    double ***src;
    double ***mu;
    double ***mu_ext;
    double h,b;
    double dx,dy,dz;

    np = density->blend->np;
    ns = density->blend->ns;
    nx = density->geometry->nx;
    ny = density->geometry->ny;
    nz = density->geometry->nz;
    dx = density->geometry->lx / density->geometry->nx;
    dy = density->geometry->ly / density->geometry->ny;
    dz = density->geometry->lz / density->geometry->nz;

    psi = density->psi;
    src = get_unused_work_array(psi,density);
    mu = potential->mu;
    mu_ext = potential->mu_ext;



    calculate_chemical_potential_for_adi(density,potential,
                                         force,omega,noise,
                                         plan,mgcondition,multigrid,
                                         use_multigrid);


        /*  evolve density field  for x axis.  */
    for(p = 0;p < np;p++)
    {
        for(i = 0;i < ns[p];i++)
        {
            b = density->blend->chain[p]->b[i];
            b = b * b / 3.0;
            h = 3.0 * dx * dx / omega / b;

            calculate_adi_source_3d_x(nx,ny,nz,
                                      dx * dx,dy * dy,dz * dz,
                                      h,omega,
                                      src[p][i],psi[p][i],
                                      mu[p][i],mu_ext[p][i]);

            set_tridiagonal_matrix(nx,h,tridiag);
            solve_cyclic_tridiag(nx,ny * nz,ny * nz,
                                 tridiag,
                                 src[p][i],u);
        }
    }
    density->psi = src;
    psi = density->psi;
    src = get_unused_work_array(psi,density);
    normalize_density_field(density);

    calculate_chemical_potential_incompressible(density->blend,density,potential);
    evolve_multiplier(potential,omega);
    correct_density_field(density);


    calculate_chemical_potential_for_adi(density,potential,
                                         force,omega,noise,
                                         plan,mgcondition,multigrid,
                                         use_multigrid);


        /*  evolve density field for y axis.  */
    for(p = 0;p < np;p++)
    {
        for(i = 0;i < ns[p];i++)
        {
            b = density->blend->chain[p]->b[i];
            b = b * b / 3.0;
            h = 3.0 * dy * dy / omega / b;

            calculate_adi_source_3d_y(nx,ny,nz,
                                      dx * dx,dy * dy,dz * dz,
                                      h,omega,
                                      src[p][i],psi[p][i],
                                      mu[p][i],mu_ext[p][i]);

            set_tridiagonal_matrix(ny,h,tridiag);
            solve_cyclic_tridiag(ny,nx * nz,nz,
                                 tridiag,
                                 src[p][i],u);
        }
    }
    density->psi = src;
    psi = density->psi;
    src = get_unused_work_array(psi,density);
    normalize_density_field(density);

    calculate_chemical_potential_incompressible(density->blend,density,potential);
    evolve_multiplier(potential,omega);
    correct_density_field(density);


    calculate_chemical_potential_for_adi(density,potential,
                                         force,omega,noise,
                                         plan,mgcondition,multigrid,
                                         use_multigrid);


        /*  evolve density field for z axis.  */
    for(p = 0;p < np;p++)
    {
        for(i = 0;i < ns[p];i++)
        {
            b = density->blend->chain[p]->b[i];
            b = b * b / 3.0;
            h = 3.0 * dz * dz / omega / b;

            calculate_adi_source_3d_z(nx,ny,nz,
                                      dx * dx,dy * dy,dz * dz,
                                      h,omega,
                                      src[p][i],psi[p][i],
                                      mu[p][i],mu_ext[p][i]);

            set_tridiagonal_matrix(nz,h,tridiag);
            solve_cyclic_tridiag(nz,nx * ny,1,
                                 tridiag,
                                 src[p][i],u);
        }
    }
    density->psi = src;
    normalize_density_field(density);

    calculate_chemical_potential_incompressible(density->blend,density,potential);
    evolve_multiplier(potential,omega);
    correct_density_field(density);
}

/*---------------------------------------------------------------------------*/
void evolve_multiplier(chemical_potential *potential,double omega)
{
    int r;
    int n;
    double *kappa,*mu_kappa;

    n = lattice_point_number(potential->geometry);
    kappa = potential->kappa;
    mu_kappa = potential->mu_kappa;

    for(r = 0;r < n;r++)
    {
        kappa[r] -= omega * mu_kappa[r];
    }
}

/*---------------------------------------------------------------------------*/
void set_tridiagonal_matrix(int n,double h,tridiag_matrix *tridiag)
{
    int i;

    for(i = 0;i < n;i++)
    {
        tridiag->dl[i] = -1;
        tridiag->du[i] = -1;
        tridiag->d[i] = h + 2;
    }
}

/*---------------------------------------------------------------------------*/
void calculate_chemical_potential_for_adi(density_field *density,
                                          chemical_potential *potential,
                                          external_force *force,
                                          double omega,double noise,
                                          field_fftw_plan *plan,
                                          multigrid_condition *mgcondition,
                                          multigrid_field *multigrid,
                                          int use_multigrid)
{
        /*  calculate chemical potential.  */
    if(use_multigrid)
        calculate_chemical_potential_except_gradient_multigrid(density->blend,density,potential,mgcondition,multigrid);
    else
        calculate_chemical_potential_except_gradient(density->blend,density,potential,plan);
    calculate_chemical_potential_external(density,potential,omega);
    add_force_to_chemical_potential(potential,force);
    add_noise_to_chemical_potential(potential,noise);
}

/*---------------------------------------------------------------------------*/
