/*---------------------------------------------------------------------------*/

/*!  \file  freeenergy.c
     \brief  calculate the free energy of the system.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005,2006 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#ifdef HAVE_MATH_H
#include <math.h>
#endif

#include "freeenergy.h"
#include "ln.h"

/*---------------------------------------------------------------------------*/
static double calculate_free_energy_longrange_after_fft(density_field *density);
static double calculate_free_energy_gradient_after_fft(density_field *density);

/*---------------------------------------------------------------------------*/
double calculate_free_energy(density_field *density,
                             field_fftw_plan *plan)
{
    double F;

    F = calculate_free_energy_longrange(density,plan);
    F += calculate_free_energy_local(density);
    F += calculate_free_energy_gradient(density,plan);

    return F;
}

/*---------------------------------------------------------------------------*/
double calculate_free_energy_longrange(density_field *density,
                                       field_fftw_plan *plan)
{
    int i,p;
    int np,*ns;
    double F;
    field_geometry *geometry;

    geometry = density->geometry;
    np = density->blend->np;
    ns = density->blend->ns;

    for(p = 0;p < np;p++)
    {
        if(ns[p] != 1)
        {
            for(i = 0;i < ns[p];i++)
            {
                if(density->blend->chain[p]->lambda > 0.0)
                {
                    operate_Li_cut(geometry,
                                   density->psi[p][i],density->Li_psi[p][i],
                                   density->blend->chain[p]->lambda,plan);
                }
                else
                {
                    operate_Li(geometry,
                               density->psi[p][i],density->Li_psi[p][i],plan);
                }
            }
        }
    }


    F = calculate_free_energy_longrange_after_fft(density);

    return F;
}

/*---------------------------------------------------------------------------*/
double calculate_free_energy_gradient(density_field *density,
                                      field_fftw_plan *plan)
{
    int i,p;
    int np,*ns;
    double F;
    field_geometry *geometry;

    geometry = density->geometry;
    np = density->blend->np;
    ns = density->blend->ns;

    for(p = 0;p < np;p++)
    {
        for(i = 0;i < ns[p];i++)
        {
            operate_L(geometry,
                      density->psi[p][i],density->L_psi[p][i]);
        }
    }


    F = calculate_free_energy_gradient_after_fft(density);

    return F;
}

/*---------------------------------------------------------------------------*/
double calculate_free_energy_longrange_after_fft(density_field *density)
{
    int p,i,j;
    int n;
    int r;
    int np,*ns;
    double ***psi,***Li_psi;
    double **A;
    double *f;
    double F;
    polymer_blend *blend;

    blend = density->blend;

    np = blend->np;
    ns = blend->ns;
    psi = density->psi;
    Li_psi = density->Li_psi;
    n = lattice_point_number(density->geometry);


    F = 0.0;
    for(p = 0;p < np;p++)
    {
        f = blend->chain[p]->f;
        A = blend->propagator[p]->A;

        for(i = 0;i < ns[p];i++)
        {
            for(j = 0;j < ns[p];j++)
            {
                for(r = 0;r < n;r++)
                {
                        /*  the long range term.  */
                    F += - (2.0 * (sqrt(f[i] * f[j]) * A[i][j])
                            * psi[p][i][r] * Li_psi[p][j][r]);
                }
            }
        }
    }
    F /= n;

    return F;
}

/*---------------------------------------------------------------------------*/
double calculate_free_energy_local(density_field *density)
{
    int p,q,i,j;
    int n;
    int r;
    int np,*ns;
    double ***psi;
    double **C,****chi;
    double *f;
    double F,F0;
    polymer_blend *blend;

    blend = density->blend;

    np = blend->np;
    ns = blend->ns;
    psi = density->psi;
    chi = blend->chi;
    n = lattice_point_number(density->geometry);


    F = 0.0;
    for(p = 0;p < np;p++)
    {
        f = blend->chain[p]->f;
        C = blend->propagator[p]->C;

        for(i = 0;i < ns[p];i++)
        {
            for(r = 0;r < n;r++)
            {
                    /*  the entropy term.  */
                F += 2.0 * (f[i] * C[i][i])
                    * psi[p][i][r] * psi[p][i][r] * LN(psi[p][i][r]);

                    /*  cancel the over count.  */
                F -= 2.0 * (f[i] * C[i][i])
                      * psi[p][i][r] * psi[p][i][r];
            }

            for(j = 0;j < ns[p];j++)
            {
                for(r = 0;r < n;r++)
                {
                        /*  the correction term.  */
                    F += + (2.0 * (sqrt(f[i] * f[j]) * C[i][j])
                            * psi[p][i][r] * psi[p][j][r]);
                }
            }

            for(q = 0;q < np;q++)
            {
                for(j = 0;j < ns[q];j++)
                {
                    for(r = 0;r < n;r++)
                    {
                            /*  the chi parameter term.  */
                        F += ((1.0 / 2.0) * chi[p][i][q][j])
                            * psi[p][i][r] * psi[p][i][r]
                            * psi[q][j][r] * psi[q][j][r];
                    }
                }
            }
        }
    }
    F /= n;


    F0 = calculate_free_energy_homogeneous(blend);

    return (F - F0);
}

/*---------------------------------------------------------------------------*/
double calculate_free_energy_gradient_after_fft(density_field *density)
{
    int p,i,j;
    int n;
    int r;
    int np,*ns;
    double ***psi,***L_psi;
    double **B;
    double *f;
    double F;
    polymer_blend *blend;

    blend = density->blend;

    np = blend->np;
    ns = blend->ns;
    psi = density->psi;
    L_psi = density->L_psi;
    n = lattice_point_number(density->geometry);


    F = 0.0;
    for(p = 0;p < np;p++)
    {
        f = blend->chain[p]->f;
        B = blend->propagator[p]->B;

        for(i = 0;i < ns[p];i++)
        {
            for(j = 0;j < ns[p];j++)
            {
                for(r = 0;r < n;r++)
                {
                        /*  the gradient term.  */
                    F += - (2.0 * (sqrt(f[i] * f[j]) * B[i][j])
                            * psi[p][i][r] * L_psi[p][j][r]);
                }
            }
        }
    }
    F /= n;

    return F;
}

/*---------------------------------------------------------------------------*/
double calculate_free_energy_homogeneous(polymer_blend *blend)
{
    int p,q,i,j;
    int np,*ns;
    double psi_bar_pi,psi_bar_pj,psi_bar_qj;
    double **C,****chi;
    double *f;
    double F0;

    np = blend->np;
    ns = blend->ns;
    chi = blend->chi;


    F0 = 0.0;
    for(p = 0;p < np;p++)
    {
        f = blend->chain[p]->f;
        C = blend->propagator[p]->C;

        for(i = 0;i < ns[p];i++)
        {
            psi_bar_pi = psi_spatial_average(blend,p,i);

                /*  the long range term and the gradient term are zero.  */

                /*  the entropy term.  */
            F0 += 2.0 * (f[i] * C[i][i])
                * psi_bar_pi * psi_bar_pi * LN(psi_bar_pi);

                    /*  cancel the over count.  */
            F0 -= 2.0 * (f[i] * C[i][i])
                * psi_bar_pi * psi_bar_pi;

            for(j = 0;j < ns[p];j++)
            {
                psi_bar_pj = psi_spatial_average(blend,p,j);

                    /*  the correction term (the long range term
                        and the gradient term will be 0).  */
                F0 += 2.0 * (sqrt(f[i] * f[j]) * C[i][j])
                    * psi_bar_pi * psi_bar_pj;
            }

            for(q = 0;q < np;q++)
            {
                for(j = 0;j < ns[q];j++)
                {
                    psi_bar_qj = psi_spatial_average(blend,q,j);

                        /*  the chi parameter term.  */
                    F0 += ((1.0 / 2.0) * chi[p][i][q][j])
                        * psi_bar_pi * psi_bar_pi
                        * psi_bar_qj * psi_bar_qj;
                }
            }
        }
    }

    return F0;
}

/*---------------------------------------------------------------------------*/
double calculate_free_energy_differential(density_field *density,
                                          field_fftw_plan *plan)
{
    double dF;
    double lx;

    lx = density->geometry->lx;

    dF = + (-2.0 / lx)
        * calculate_free_energy_longrange(density,plan)
        + (2.0 / lx)
        * calculate_free_energy_gradient(density,plan);

    return dF;
}

/*---------------------------------------------------------------------------*/
double calculate_free_energy_differential2(density_field *density,
                                          field_fftw_plan *plan)
{
    double dF;
    double lx,lx2;

    lx = density->geometry->lx;
    lx2 = lx * lx;

    dF = + (6.0 / lx2)
        * calculate_free_energy_longrange(density,plan)
        + (2.0 / lx2)
        * calculate_free_energy_gradient(density,plan);

    return dF;
}

/*---------------------------------------------------------------------------*/
double calculate_optimal_lattice_size(density_field *density,
                                      field_fftw_plan *plan)
{
    double lx_opt;
    double lx;

    lx = density->geometry->lx;

    lx_opt = sqrt(sqrt
              (calculate_free_energy_longrange(density,plan)
               / calculate_free_energy_gradient(density,plan)))
        * lx;

    return lx_opt;
}

/*---------------------------------------------------------------------------*/
