/*---------------------------------------------------------------------------*/

/*!  \file  gpwrite.c
     \brief  output the field data as gnuplot format.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005,2006,2018 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#include <stdio.h>
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifdef HAVE_ZLIB_H
#include <zlib.h>
#endif

#include "gpwrite.h"

/*---------------------------------------------------------------------------*/
static void write_density_plot_1d(char *filename,
                                  density_field *density);
static void write_density_plot_2d(char *filename,
                                  density_field *density);
static void write_density_plot_3d(char *filename,
                                  density_field *density);
static void write_chemical_potential_plot_1d(char *filename,
                                             chemical_potential *potential);
static void write_chemical_potential_plot_2d(char *filename,
                                             chemical_potential *potential);
static void write_chemical_potential_plot_3d(char *filename,
                                             chemical_potential *potential);

/*---------------------------------------------------------------------------*/
void write_density_plot(char *filename,
                        density_field *density)
{
    if((filename == NULL) || (strlen(filename) == 0))
        return;

    switch(density->geometry->dimension)
    {
        case 1:
            write_density_plot_1d(filename,density);
            break;

        case 2:
            write_density_plot_2d(filename,density);
            break;

        case 3:
            write_density_plot_3d(filename,density);
            break;

        default:
            return;
    }
}

/*---------------------------------------------------------------------------*/
void write_chemical_potential_plot(char *filename,
                                   chemical_potential *potential)
{
    if((filename == NULL) || (strlen(filename) == 0))
        return;

    switch(potential->geometry->dimension)
    {
        case 1:
            write_chemical_potential_plot_1d(filename,potential);
            break;

        case 2:
            write_chemical_potential_plot_2d(filename,potential);
            break;

        case 3:
            write_chemical_potential_plot_3d(filename,potential);
            break;

        default:
            return;
    }
}

/*---------------------------------------------------------------------------*/
void write_density_plot_1d(char *filename,
                           density_field *density)
{
    gzFile fp;
    int p,i,x;
    int np,*ns;
    int nx;
    double lx;
    double ***phi;

    np = density->blend->np;
    ns = density->blend->ns;
    nx = density->geometry->nx;
    lx = density->geometry->lx;
    phi = density->phi;

    fp = gzopen(filename,"wb");

    for(x = 0;x < nx;x++)
    {
        gzprintf(fp,"%g ",((double)x * lx / nx));
        for(p = 0;p < np;p++)
        {
            for(i = 0;i < ns[p];i++)
            {
                gzprintf(fp,"%g ",phi[p][i][x]);
            }
        }
        gzprintf(fp,"\n");
    }

    gzclose(fp);
}

/*---------------------------------------------------------------------------*/
void write_density_plot_2d(char *filename,
                           density_field *density)
{
    gzFile fp;
    int p,i,x,y;
    int np,*ns;
    int nx,ny;
    double lx,ly;
    double ***phi;

    np = density->blend->np;
    ns = density->blend->ns;
    nx = density->geometry->nx;
    ny = density->geometry->ny;
    lx = density->geometry->lx;
    ly = density->geometry->ly;
    phi = density->phi;

    fp = gzopen(filename,"wb");

    for(x = 0;x < nx;x++)
    {
        for(y = 0;y < ny;y++)
        {
            gzprintf(fp,"%g %g ",((double)x * lx / nx),((double)y * ly / ny));
            for(p = 0;p < np;p++)
            {
                for(i = 0;i < ns[p];i++)
                {
                    gzprintf(fp,"%g ",phi[p][i][x * ny + y]);
                }
            }
            gzprintf(fp,"\n");
        }
        gzprintf(fp,"\n");
    }

    gzclose(fp);
}

/*---------------------------------------------------------------------------*/
void write_density_plot_3d(char *filename,
                           density_field *density)
{
    gzFile fp;
    int p,i,x,y,z;
    int np,*ns;
    int nx,ny,nz;
    double lx,ly,lz;
    double ***phi;

    np = density->blend->np;
    ns = density->blend->ns;
    nx = density->geometry->nx;
    ny = density->geometry->ny;
    nz = density->geometry->nz;
    lx = density->geometry->lx;
    ly = density->geometry->ly;
    lz = density->geometry->lz;
    phi = density->phi;

    fp = gzopen(filename,"wb");

    for(x = 0;x < nx;x++)
    {
        for(y = 0;y < ny;y++)
        {
            for(z = 0;z < nz;z++)
            {
                gzprintf(fp,"%g %g %g ",((double)x * lx / nx),
                        ((double)y * ly / ny),((double)z * lz / nz));
                for(p = 0;p < np;p++)
                {
                    for(i = 0;i < ns[p];i++)
                    {
                        gzprintf(fp,"%g ",phi[p][i][(x * ny + y) * nz + z]);
                    }
                }
                gzprintf(fp,"\n");
            }
            gzprintf(fp,"\n");
        }
        gzprintf(fp,"\n");
    }

    gzclose(fp);
}

/*---------------------------------------------------------------------------*/
void write_chemical_potential_plot_1d(char *filename,
                                      chemical_potential *potential)
{
    gzFile fp;
    int p,i,x;
    int np,*ns;
    int nx;
    double lx;
    double ***mu;

    np = potential->blend->np;
    ns = potential->blend->ns;
    nx = potential->geometry->nx;
    lx = potential->geometry->lx;
    mu = potential->mu;

    fp = gzopen(filename,"wb");

    for(x = 0;x < nx;x++)
    {
        gzprintf(fp,"%g ",((double)x * lx / nx));
        for(p = 0;p < np;p++)
        {
            for(i = 0;i < ns[p];i++)
            {
                gzprintf(fp,"%g ",mu[p][i][x] * mu[p][i][x]);
            }
        }
        gzprintf(fp,"\n");
    }

    gzclose(fp);
}

/*---------------------------------------------------------------------------*/
void write_chemical_potential_plot_2d(char *filename,
                                      chemical_potential *potential)
{
    gzFile fp;
    int p,i,x,y;
    int np,*ns;
    int nx,ny;
    double lx,ly;
    double ***mu;

    np = potential->blend->np;
    ns = potential->blend->ns;
    nx = potential->geometry->nx;
    ny = potential->geometry->ny;
    lx = potential->geometry->lx;
    ly = potential->geometry->ly;
    mu = potential->mu;

    fp = gzopen(filename,"wb");

    for(x = 0;x < nx;x++)
    {
        for(y = 0;y < ny;y++)
        {
            gzprintf(fp,"%g %g ",((double)x * lx / nx),((double)y * ly / ny));
            for(p = 0;p < np;p++)
            {
                for(i = 0;i < ns[p];i++)
                {
                    gzprintf(fp,"%g ",mu[p][i][x * ny + y] * mu[p][i][x * ny + y]);
                }
            }
            gzprintf(fp,"\n");
        }
        gzprintf(fp,"\n");
    }

    gzclose(fp);
}

/*---------------------------------------------------------------------------*/
void write_chemical_potential_plot_3d(char *filename,
                                      chemical_potential *potential)
{
    gzFile fp;
    int p,i,x,y,z;
    int np,*ns;
    int nx,ny,nz;
    double lx,ly,lz;
    double ***mu;

    np = potential->blend->np;
    ns = potential->blend->ns;
    nx = potential->geometry->nx;
    ny = potential->geometry->ny;
    nz = potential->geometry->nz;
    lx = potential->geometry->lx;
    ly = potential->geometry->ly;
    lz = potential->geometry->lz;
    mu = potential->mu;

    fp = gzopen(filename,"wb");

    for(x = 0;x < nx;x++)
    {
        for(y = 0;y < ny;y++)
        {
            for(z = 0;z < nz;z++)
            {
                gzprintf(fp,"%g %g %g ",((double)x * lx / nx),
                        ((double)y * ly / ny),((double)z * lz / nz));
                for(p = 0;p < np;p++)
                {
                    for(i = 0;i < ns[p];i++)
                    {
                        gzprintf(fp,"%g ",mu[p][i][(x * ny + y) * nz + z] * mu[p][i][(x * ny + y) * nz + z]);
                    }
                }
                gzprintf(fp,"\n");
            }
            gzprintf(fp,"\n");
        }
        gzprintf(fp,"\n");
    }

    gzclose(fp);
}

/*---------------------------------------------------------------------------*/
