/*---------------------------------------------------------------------------*/

/*!  \file  multigrid.c
     \brief  the multigrid infomation for the field.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005,2006 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_MATH_H
#include <math.h>
#endif

#include "xmalloc.h"
#include "multigrid.h"

/*---------------------------------------------------------------------------*/
static int grid_level(int n);
static int grid_level_for_geometry(field_geometry *geometry);
static void set_multigrid(multigrid_field *multigrid,
                          field_geometry *geometry);

/*---------------------------------------------------------------------------*/
multigrid_field *allocate_multigrid_field(field_geometry *geometry)
{
    multigrid_field *multigrid;
    int level;

    multigrid = xmalloc(sizeof(multigrid_field));
    multigrid->dimension = geometry->dimension;
    multigrid->lx = geometry->lx;
    multigrid->ly = geometry->ly;
    multigrid->lz = geometry->lz;

    level = grid_level_for_geometry(geometry);
    multigrid->level = level;

    multigrid->nx = xmalloc(sizeof(int) * level);
    multigrid->ny = xmalloc(sizeof(int) * level);
    multigrid->nz = xmalloc(sizeof(int) * level);

    multigrid->u = xmalloc(sizeof(double *) * level);
    multigrid->v = xmalloc(sizeof(double *) * level);
    multigrid->f = xmalloc(sizeof(double *) * level);
    multigrid->r = xmalloc(sizeof(double *) * level);

    set_multigrid(multigrid,geometry);

    return multigrid;
}

/*---------------------------------------------------------------------------*/
void free_multigrid_field(multigrid_field *multigrid)
{
    int i;

    if(multigrid == NULL)
        return;

    for(i = 0;i < multigrid->level;i++)
    {
        free(multigrid->u[i]);
        free(multigrid->v[i]);
        free(multigrid->f[i]);
        free(multigrid->r[i]);
    }

    free(multigrid->u);
    free(multigrid->v);
    free(multigrid->f);
    free(multigrid->r);

    switch(multigrid->dimension)
    {
        case 1:
            free(multigrid->nx);
            break;

        case 2:
            free(multigrid->nx);
            free(multigrid->ny);
            break;

        case 3:
            free(multigrid->nx);
            free(multigrid->ny);
            free(multigrid->nz);
            break;

        default:
            break;
    }

    free(multigrid);
}

/*---------------------------------------------------------------------------*/
int lattice_point_number_multigrid(multigrid_field *multigrid,int level)
{
    int n;

    if((level < 0) || (multigrid->level <= level))
        return 0;

    switch(multigrid->dimension)
    {
        case 1:
            n = multigrid->nx[level];
            break;

        case 2:
            n = multigrid->nx[level] * multigrid->ny[level];
            break;

        case 3:
            n = multigrid->nx[level] * multigrid->ny[level] * multigrid->nz[level];
            break;

        default:
            n = 0;
            break;
    }

    return n;
}

/*---------------------------------------------------------------------------*/
int grid_level(int n)
{
    int m;

    m = 1;

    while((n = (n + 1) / 2) != 1)
        m++;
    return m;
}

/*---------------------------------------------------------------------------*/
int grid_level_for_geometry(field_geometry *geometry)
{
    int lx,ly,lz;
    int level;

    switch(geometry->dimension)
    {
        case 1:
            level = grid_level(geometry->nx);
            break;

        case 2:
            lx = grid_level(geometry->nx);
            ly = grid_level(geometry->ny);

            level = lx;
            if(ly > lx)
                level = ly;
            break;

        case 3:
            lx = grid_level(geometry->nx);
            ly = grid_level(geometry->ny);
            lz = grid_level(geometry->nz);
            level = lx;
            if(ly > lx)
                level = ly;
            if(lz > level)
                level = lz;
            break;

        default:
            return 0;
    }

    level++;

    return level;
}

/*---------------------------------------------------------------------------*/
void set_multigrid(multigrid_field *multigrid,
                   field_geometry *geometry)
{
    int i,n;

    switch(multigrid->dimension)
    {
        case 1:
            multigrid->nx[0] = geometry->nx;
            for(i = 1;i < multigrid->level;i++)
            {
                multigrid->nx[i] = (multigrid->nx[i - 1] + 1) / 2;
            }
            break;

        case 2:
            multigrid->nx[0] = geometry->nx;
            multigrid->ny[0] = geometry->ny;
            for(i = 1;i < multigrid->level;i++)
            {
                multigrid->nx[i] = (multigrid->nx[i - 1] + 1) / 2;
                multigrid->ny[i] = (multigrid->ny[i - 1] + 1) / 2;
            }
            break;

        case 3:
            multigrid->nx[0] = geometry->nx;
            multigrid->ny[0] = geometry->ny;
            multigrid->nz[0] = geometry->nz;
            for(i = 1;i < multigrid->level;i++)
            {
                multigrid->nx[i] = (multigrid->nx[i - 1] + 1) / 2;
                multigrid->ny[i] = (multigrid->ny[i - 1] + 1) / 2;
                multigrid->nz[i] = (multigrid->nz[i - 1] + 1) / 2;
            }
            break;

        default:
            return;
    }

    for(i = 0;i < multigrid->level;i++)
    {
        n = lattice_point_number_multigrid(multigrid,i);
        multigrid->u[i] = xmalloc(sizeof(double) * n);
        multigrid->v[i] = xmalloc(sizeof(double) * n);
        multigrid->f[i] = xmalloc(sizeof(double) * n);
        multigrid->r[i] = xmalloc(sizeof(double) * n);
    }
}

/*---------------------------------------------------------------------------*/
