/*---------------------------------------------------------------------------*/

/*!  \file  operator.h
     \brief  perform the field operator to the field  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005-2007 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_OPERATOR_H
#define INCLUDE_OPERATOR_H

#ifdef HAVE_FFTW3_H
#include <fftw3.h>
#endif

#include "geometry.h"

/*---------------------------------------------------------------------------*/
/*!  \brief  plan for FFTW.  */
typedef struct field_fftw_plan
{
        /*!  \brief  geometry of the field.  */
    field_geometry *geometry;

        /*!  \brief  forward FFTW plan.  */
    fftw_plan plan_forward;
        /*!  \brief  backward FFT plan.  */
    fftw_plan plan_backward;

        /*!  \brief  work array.  */
    double *work;
        /*!  \brief  work array.  */
    fftw_complex *F_work;
} field_fftw_plan;

/*---------------------------------------------------------------------------*/
/*!  \brief  allocate a new FFTW plan.
     \param  geometry  geometry for the field
     \param  wisdom_file  wisdom file used by FFTW  */
field_fftw_plan *allocate_field_fftw_plan(field_geometry *geometry,
                                          char *wisdom_file);
/*!  \brief  free a FFTW plan.
     \param  plan  FFTW plan to free  */
void free_field_fftw_plan(field_fftw_plan *plan);

/*!  \brief  operate Laplacian inverse to a field.
     \param  geometry  geometry for the field
     \param  psi  field
     \param  Li_psi Laplacian inverse of \a psi
     \param  plan  plan for FFTW  */
void operate_Li(field_geometry *geometry,double *psi,double *Li_psi,
                field_fftw_plan *plan);
/*!  \brief  operate Laplacian inverse to a field.
     \param  geometry  geometry for the field
     \param  psi  field
     \param  Li_psi Laplacian inverse of \a psi
     \param  plan  plan for FFTW  */
void operate_Li_cut(field_geometry *geometry,double *psi,double *Li_psi,
                    double lambda,field_fftw_plan *plan);
/*!  \brief  operate Laplacian inverse to a field (set offset value for statics).
     \param  geometry  geometry for the field
     \param  psi  field
     \param  Li_psi Laplacian inverse of \a psi
     \param  plan  plan for FFTW  */
void operate_Li_cut_for_statics(field_geometry *geometry,double *psi,double *Li_psi,
                                double lambda,field_fftw_plan *plan);
/*!  \brief  operate Laplacian to a field.
     \param  geometry  geometry for the field
     \param  psi  field
     \param  Li_psi Laplacian of \a psi  */
void operate_L(field_geometry *geometry,double *psi,double *L_psi);

/*!  \brief  operate Fourier transformation to a field.
     \param  geometry  geometry for the field
     \param  psi  field
     \param  F_psi  Fourier transform of \a psi
     \param  plan  plan for FFTW  */
void operate_F(field_geometry *geometry,double *psi,fftw_complex *F_psi,
               field_fftw_plan *plan);
/*!  \brief  operate inverse Fourier transformation to a field.
     \param  geometry  geometry for the field
     \param  F_psi  field
     \param  psi  inverse Fourier transform of \a F_psi
     \param  plan  plan for FFTW  */
void operate_Fi(field_geometry *geometry,fftw_complex *F_psi,double *psi,
                field_fftw_plan *plan);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
