/*---------------------------------------------------------------------------*/

/*!  \file  propagator.c
     \brief  calculate the free propagator for Gaussian chains.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005,2006 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_MATH_H
#include <math.h>
#endif

#include "xmalloc.h"
#include "matrix.h"
#include "lu.h"
#include "correlation.h"
#include "propagator.h"

#define INF (-1.0)
#define ISINF(x) ((x) < 0)

/*---------------------------------------------------------------------------*/
static void calculate_A(polymer_chain *chain,double **Hi,double **A);
static void calculate_B(polymer_chain *chain,double **B);
static void calculate_q2star(polymer_chain *chain,double **A,double **B,
                             double **q2star);
static void calculate_C0(polymer_chain *chain,double **Hi,double **H2,
                          double **A,double **C0);
static void calculate_C(polymer_chain *chain,double **q2star,double **Hi2,
                         double **C0,double **A,double **B,double **C);
static void calculate_hstar(polymer_chain *chain,double q2,double **hstar);

/*---------------------------------------------------------------------------*/
void calculate_chain_propagator(polymer_chain *chain,
                                chain_propagator *propagator)
{
    int n;
    double **H1,**Hi,**H2,**Hi2,**C0;
    double **q2star;
    double **A,**C,**B;

    n = chain->ns;
    A = propagator->A;
    C = propagator->C;
    B = propagator->B;

    H1 = allocate_matrix(n);
    Hi = allocate_matrix(n);
    H2 = allocate_matrix(n);
    Hi2 = allocate_matrix(n);
    C0 = allocate_matrix(n);
    q2star = allocate_matrix(n);

        /*  set H1.  */
    calculate_H01(chain,H1);
        /*  calculate H1 inverse.  */
    inverse(H1,Hi,n);

        /*  calculate H2.  */
    calculate_H02(chain,H2);

        /*  calculate Hi2.  */
    calculate_Hi2(chain,Hi2);

        /*  calculate A,B.  */
    calculate_A(chain,Hi,A);
    calculate_B(chain,B);

        /*  calculate q2star.  */
    calculate_q2star(chain,A,B,q2star);
        /*  calculate C0.  */
    calculate_C0(chain,Hi,H2,A,C0);

        /*  calculate C.  */
    calculate_C(chain,q2star,Hi2,C0,A,B,C);


    free_matrix(H1);
    free_matrix(Hi);
    free_matrix(H2);
    free_matrix(Hi2);
    free_matrix(C0);
    free_matrix(q2star);
}

/*---------------------------------------------------------------------------*/
chain_propagator *allocate_chain_propagator(int n)
{
    chain_propagator *propagator;

    propagator = xmalloc(sizeof(chain_propagator));

    propagator->n = n;
    propagator->A = allocate_matrix(n);
    propagator->C = allocate_matrix(n);
    propagator->B = allocate_matrix(n);

    return propagator;
}

/*---------------------------------------------------------------------------*/
void free_chain_propagator(chain_propagator *propagator)
{
    if(propagator == NULL)
        return;

    free_matrix(propagator->A);
    free_matrix(propagator->C);
    free_matrix(propagator->B);

    free(propagator);
}

/*---------------------------------------------------------------------------*/
static void calculate_A(polymer_chain *chain,double **Hi,double **A)
{
    int i,j,k,l,n;
    double *f;
    double fHf;
    double **HffH;

    n = chain->ns;
    f = chain->f;

    if(n == 1)
    {
            /*  for homopolymers.  */
        A[0][0] = 0.0;
        return;
    }


        /*  for block copolymers.  */

    HffH = allocate_matrix(n);

    for(i = 0;i < n;i++)
    {
        for(j = 0;j < n;j++)
        {
            HffH[i][j] = 0;
            for(k = 0;k < n;k++)
            {
                for(l = 0;l < n;l++)
                {
                    HffH[i][j] += Hi[i][k] * f[k] * f[l] * Hi[l][j];
                }
            }
        }
    }
    fHf = 0;
    for(i = 0;i < n;i++)
    {
        for(j = 0;j < n;j++)
        {
            fHf += f[i] * Hi[i][j] * f[j];
        }
    }

    for(i = 0;i < n;i++)
    {
        for(j = 0;j < n;j++)
        {
            A[i][j] = -Hi[i][j] + (HffH[i][j] / fHf);
        }
    }

    free_matrix(HffH);
}

/*---------------------------------------------------------------------------*/
static void calculate_B(polymer_chain *chain,double **B)
{
    int i,j,n;
    double *f,*b;

    n = chain->ns;
    f = chain->f;
    b = chain->b;

    for(i = 0;i < n;i++)
    {
        for(j = 0;j < n;j++)
        {
            B[i][j] = 0;
        }
        B[i][i] = (b[i] * b[i]) / (12.0 * f[i]);
    }
}

/*---------------------------------------------------------------------------*/
static void calculate_q2star(polymer_chain *chain,double **A,double **B,
                             double **q2star)
{
    int i,j,n;

    n = chain->ns;

    for(i = 0;i < n;i++)
    {
        for(j = 0;j < n;j++)
        {
            if(B[i][j] == 0)
            {
                q2star[i][j] = INF;
            }
            else
            {
                q2star[i][j] = sqrt(A[i][j] / B[i][j]);
            }
        }
    }
}

/*---------------------------------------------------------------------------*/
static void calculate_C0(polymer_chain *chain,double **Hi,double **H2,
                          double **A,double **C0)
{
    int i,j,k,l,n;
    double N,*f;
    double fHf;
    double **GHG;

    n = chain->ns;
    N = chain->N;
    f = chain->f;

    GHG = allocate_matrix(n);

    for(i = 0;i < n;i++)
    {
        for(j = 0;j < n;j++)
        {
            GHG[i][j] = 0;
            for(k = 0;k < n;k++)
            {
                for(l = 0;l < n;l++)
                {
                    GHG[i][j] += A[i][k] * H2[k][l] * A[l][j];
                }
            }
        }
    }
    fHf = 0;
    for(i = 0;i < n;i++)
    {
        for(j = 0;j < n;j++)
        {
            fHf += f[i] * Hi[i][j] * f[j];
        }
    }

    for(i = 0;i < n;i++)
    {
        for(j = 0;j < n;j++)
        {
            C0[i][j] = -GHG[i][j] + (A[i][j] + Hi[i][j]) / (N * fHf);
        }
    }

    free_matrix(GHG);
}

/*---------------------------------------------------------------------------*/
static void calculate_C(polymer_chain *chain,double **q2star,double **H2,
                         double **C0,double **A,double **B,double **C)
{
    int i,j,n;
    double *f,*b,**l2;
    double **hstar,**hstari;

    n = chain->ns;
    f = chain->f;
    b = chain->b;
    l2 = chain->l2;

    if(n == 1)
    {
            /*  for homopolymers  */
        C[0][0] = C0[0][0];
        return;
    }

        /*  for block copolymers  */
    hstar = allocate_matrix(n);
    hstari = allocate_matrix(n);



        /*  determine C from q2star.  */
    for(i = 0;i < n;i++)
    {
        for(j = 0;j < n;j++)
        {
            if((n == 1) || (q2star == 0))
            {
                    /*  only for homopolymers  */
                C[i][j] = C0[i][j];
            }
            else if(ISINF(q2star[i][j]))
            {
                if(l2[i][j] != 0.0)
                {
                    C[i][j] = 0.0;
                }
                else
                {
                    C[i][j] = - (b[i] * b[i]) * (b[j] * b[j])
                        * H2[i][j] / (144.0 * f[i] * f[j]);
                }
            }
            else
            {
                    /*  calculate hstar and invert it.  */
                calculate_hstar(chain,q2star[i][j],hstar);
                inverse(hstar,hstari,n);

                C[i][j] = hstari[i][j]
                    - A[i][j] / (q2star[i][j])
                    - B[i][j] * q2star[i][j];
            }
        }
    }

    free_matrix(hstar);
    free_matrix(hstari);
}

/*---------------------------------------------------------------------------*/
static void calculate_hstar(polymer_chain *chain,double q2,double **hstar)
{
    int i,j,n;

    n = chain->ns;

    for(i = 0;i < n;i++)
    {
        for(j = 0;j < n;j++)
        {
            hstar[i][j] = h2(chain,i,j,q2);
        }

        hstar[i][i] = h1(chain,i,q2);
    }
}

/*---------------------------------------------------------------------------*/
