/*---------------------------------------------------------------------------*/

/*!  \file  tridiag.c
     \brief  solver for tridiagonal and cyclic tridiagonal matrices  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005,2006 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include "xmalloc.h"
#include "matrix.h"
#include "tridiag.h"

/*---------------------------------------------------------------------------*/
tridiag_matrix *allocate_tridiag_matrix(int n)
{
    tridiag_matrix *tridiag;

    tridiag = xmalloc(sizeof(tridiag_matrix));

    tridiag->d = allocate_vector(n);
    tridiag->dl = allocate_vector(n);
    tridiag->du = allocate_vector(n);

    return tridiag;
}

/*---------------------------------------------------------------------------*/
void free_tridiag_matrix(tridiag_matrix *tridiag)
{
    free_vector(tridiag->d);
    free_vector(tridiag->dl);
    free_vector(tridiag->du);

    free(tridiag);
}

/*---------------------------------------------------------------------------*/
void solve_tridiag(int n,int n_rhs,int n_stride,
                   tridiag_matrix *tridiag,double *b)
{
    int i,j,k;
    int n1,n2;
    int i_rhs;
    double t;
    double *d,*dl,*du;

    d = tridiag->d;
    dl = tridiag->dl;
    du = tridiag->du;

    n1 = n_stride;
    n2 = n_rhs / n_stride;

        /*  LU decomposition and forward substitution.
            This process overwrites d[i].  */
    for(i = 0;i < (n - 1);i++)
    {
        t = dl[i] / d[i];
        d[i + 1] -= t * du[i];

        for(k = 0;k < n2;k++)
        {
            for(j = 0;j < n1;j++)
            {
                i_rhs = j + k * n * n1;
                b[i_rhs + (i + 1) * n_stride] -= t * b[i_rhs + i * n_stride];
            }
        }
    }

    for(k = 0;k < n2;k++)
    {
        for(j = 0;j < n1;j++)
        {
            i_rhs = j + k * n * n1;
            b[i_rhs + i * n_stride] /= d[i];
        }
    }

        /*  Backward substitution.
            The result will be overwritten into b[i].  */
    for(i--;i >= 0;i--)
    {
        for(k = 0;k < n2;k++)
        {
            for(j = 0;j < n1;j++)
            {
                i_rhs = j + k * n * n1;
                b[i_rhs + i * n_stride] =
                    (b[i_rhs + i * n_stride]
                     - du[i] * b[i_rhs + (i + 1) * n_stride]) / d[i];
            }
        }
    }
}

/*---------------------------------------------------------------------------*/
void solve_cyclic_tridiag(int n,int n_rhs,int n_stride,
                          tridiag_matrix *tridiag,double *b,double *u)
{
    int i,j,k;
    int n1,n2;
    int i_rhs;
    double t;
    double *d,*dl,*du;
    double vy,vz;
    double vfact;

    d = tridiag->d;
    dl = tridiag->dl;
    du = tridiag->du;

    n1 = n_stride;
    n2 = n_rhs / n_stride;


        /*  Prepare vector u and modify the input matrix
            for Sherman-Morrison formula.  */
    u[0] = -d[0];
    for(i = 1;i < (n - 1);i++)
    {
        u[i] = 0;
    }
    u[n - 1] = du[n - 1];

    d[0] -= u[0];
    vfact = dl[n - 1] / u[0];
    d[n - 1] -= u[n - 1] * vfact;


        /*  LU decomposition and forward substitution.
            This process overwrites d[i].  */
    for(i = 0;i < (n - 1);i++)
    {
        t = dl[i] / d[i];
        d[i + 1] -= t * du[i];

        for(k = 0;k < n2;k++)
        {
            for(j = 0;j < n1;j++)
            {
                i_rhs = j + k * n * n1;
                b[i_rhs + (i + 1) * n_stride] -= t * b[i_rhs + i * n_stride];
            }
        }
        u[i + 1] -= t * u[i];
    }

    for(k = 0;k < n2;k++)
    {
        for(j = 0;j < n1;j++)
        {
            i_rhs = j + k * n * n1;
            b[i_rhs + i * n_stride] /= d[i];
        }
    }
    u[i] /= d[i];


        /*  Backward substitution.
            The result will be overwritten into b[i].  */
    for(i--;i >= 0;i--)
    {
        for(k = 0;k < n2;k++)
        {
            for(j = 0;j < n1;j++)
            {
                i_rhs = j + k * n * n1;
                b[i_rhs + i * n_stride] =
                    (b[i_rhs + i * n_stride]
                     - du[i] * b[i_rhs + (i + 1) * n_stride]) / d[i];
            }
        }
        u[i] = (u[i] - du[i] * u[i + 1]) / d[i];
    }


        /*  Sherman-Morrison forumula.  */
    vz = u[0] + u[n - 1] * vfact;

    for(k = 0;k < n2;k++)
    {
        for(j = 0;j < n1;j++)
        {
            i_rhs = j + k * n * n1;

            vy =  b[i_rhs] + b[i_rhs + (n - 1) * n_stride] * vfact;
            t = vy / (1 + vz);

            for(i = 0;i < n;i++)
            {
                b[i_rhs + i * n_stride] -= t * u[i];
            }
        }
    }
}

/*---------------------------------------------------------------------------*/
