/*---------------------------------------------------------------------------*/

/*!  \file  drops-dx2text.c
     \brief  utility to convert a DX output file into a text file. */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS 0
#endif
#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1
#endif

#include "getopt.h"
#include "geometry.h"
#include "dxread.h"
#include "field.h"
#include "textwrite.h"

#if defined(DX2PSI)
#define FIELDNAME "psi"
#define FIELDTEXT "psi"
#elif defined(DX2PHI)
#define FIELDNAME "phi"
#define FIELDTEXT "density"
#elif defined(DX2MU)
#define FIELDNAME "mu"
#define FIELDTEXT "chemical potential"
#else
#error DX2PSI, DX2PHI, or DX2MU must be defined.
#endif

/*---------------------------------------------------------------------------*/
static char *programstring = "drops-dx2" FIELDNAME;

static struct option longopts[] =
{
    {"input", required_argument, NULL, 'i'},
    {"output", required_argument, NULL, 'o'},
    {"version", no_argument, NULL, 'v'},
    {"help", no_argument, NULL, 'h'},
    {0, 0, NULL, 0}
};

static char *optstring = "i:o:vh";

static char *helpstring =
"\
Usage: %s [options] [input] [output]\n\
Options:\n\
  -i, --input <file>       read the DX data generated by drops from <file>\n\
  -o, --output <file>      write the output " FIELDTEXT " field to <file>\n\
  -v, --version            output version information and exit\n\
  -h, --help               display this help and exit\n\
";

static char *copyingstring =
"\
%s (%s) %s\n\
\n\
Copyright (C) 2007 Takashi Uneyama\n\
This is free software; see the source for copying conditions.  There is NO\n\
warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
";

/*---------------------------------------------------------------------------*/
static int process_option(int argc,char **argv,char **input_dx,
                          char **output_text);

/*---------------------------------------------------------------------------*/
int main(int argc,char **argv)
{
    char *input_dx = NULL;
    char *output_text = NULL;
    field_geometry *geometry;
    double **phi,**mu;
    int n;

    if(process_option(argc,argv,&input_dx,&output_text) != 0)
    {
        return EXIT_FAILURE;
    }

    if(input_dx == NULL)
    {
        fprintf(stderr,"error: no input file is specified.\n");
        return EXIT_FAILURE;
    }
    if(output_text == NULL)
    {
        fprintf(stderr,"error: no output file is specified.\n");
        return EXIT_FAILURE;
    }

    geometry = allocate_field_geometry();


    if(read_field_dx(input_dx,geometry,&phi,&mu,&n) != 0)
    {
        return EXIT_FAILURE;
    }

    if(n <= 0)
    {
        fprintf(stderr,"error: no vailid field is found in `%s'.\n",input_dx);
    }

#if defined(DX2PHI)
    write_field_text_with_lattice(output_text,geometry,phi,n);
#elif defined(DX2MU)
    write_field_text_with_lattice(output_text,geometry,mu,n);
#elif defined(DX2PSI)
    convert_field_sqrt_double(phi,geometry,n);
    write_field_text(output_text,geometry,phi,n);
#endif

    free_field_double(phi);
    free_field_double(mu);
    free_field_geometry(geometry);

    return EXIT_SUCCESS;
}

/*---------------------------------------------------------------------------*/
int process_option(int argc,char **argv,char **input_dx,char **output_text)
{
    int c;
    int option_index;
    char *program_name;

    program_name = argv[0];

    do
    {
        option_index = 0;
        c = getopt_long(argc,argv,optstring,longopts,&option_index);

        switch(c)
        {
            case 'i':
                    /*  input DX file  */
                *input_dx = optarg;
                break;

            case 'o':
                    /*  output text file  */
                *output_text = optarg;
                break;

            case 'v':
                    /*  version  */
                fprintf(stderr,copyingstring,programstring,PACKAGE,VERSION);
                exit(EXIT_SUCCESS);
                break;

            case 'h':
                    /*  help  */
                fprintf(stderr,helpstring,program_name);
                exit(EXIT_SUCCESS);
                break;

            case '?':
                    /*  invalid option  */
                exit(EXIT_FAILURE);
                break;

            case -1:
                break;

            default:
                fprintf(stderr,
                        "error: getopt returned character code 0%o\n", c);
                return 1;
        }
    }
    while(c != -1);


    if((optind < argc) && (*input_dx == NULL))
    {
            /*  input DX file  */
        *input_dx = argv[optind++];
    }
    if((optind < argc) && (*output_text == NULL))
    {
            /*  output text file  */
        *output_text = argv[optind++];
    }

    return 0;
}

/*---------------------------------------------------------------------------*/
