/*---------------------------------------------------------------------------*/

/*!  \file  drops-phi2psi.c
     \brief  utility to convert a density field file into psi field file. */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007,2018 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_MATH_H
#include <math.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif
#ifdef HAVE_CTYPE_H
#include <ctype.h>
#endif

#ifdef HAVE_ZLIB_H
#include <zlib.h>
#endif

#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS 0
#endif
#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1
#endif

#ifndef HAVE_ISSPACE
#define isspace(c) (((c) == ' ') || ((c) == '\f') || ((c) == '\n') \
                    || ((c) == '\r') || ((c) == '\t') || ((c) == '\v'))
#endif

#define ZGETS_BUFFERSIZE 0x200

#include "getopt.h"
#include "xmalloc.h"

/*---------------------------------------------------------------------------*/
static char *programstring = "drops-phi2psi";

static struct option longopts[] =
{
    {"input", required_argument, NULL, 'i'},
    {"output", required_argument, NULL, 'o'},
    {"dimension", required_argument, NULL, 'd'},
    {"version", no_argument, NULL, 'v'},
    {"help", no_argument, NULL, 'h'},
    {0, 0, NULL, 0}
};

static char *optstring = "i:o:d:vh";

static char *helpstring =
"\
Usage: %s [options] [input] [output] \n\
Options:\n\
  -i, --input <file>       read the input density field from <file>\n\
  -o, --output <file>      write the output psi field to <file>\n\
  -d, --dimension <d>      specify the dimension of the system\n\
  -v, --version            output version information and exit\n\
  -h, --help               display this help and exit\n\
";

static char *copyingstring =
"\
%s (%s) %s\n\
\n\
Copyright (C) 2007 Takashi Uneyama\n\
This is free software; see the source for copying conditions.  There is NO\n\
warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
";

/*---------------------------------------------------------------------------*/
static int process_option(int argc,char **argv,char **input_density,
                          char **output_psi,int *dimension);
static void convert_phi_to_psi(char *input_density,char *output_psi,
                               int dimension);

/*---------------------------------------------------------------------------*/
int main(int argc,char **argv)
{
    char *input_density = NULL;
    char *output_psi = NULL;
    int dimension = 0;

    if(process_option(argc,argv,&input_density,&output_psi,&dimension) != 0)
    {
        return EXIT_FAILURE;
    }

    if(input_density == NULL)
    {
        fprintf(stderr,"error: no input file is specified.\n");
        return EXIT_FAILURE;
    }
    if(output_psi == NULL)
    {
        fprintf(stderr,"error: no output file is specified.\n");
        return EXIT_FAILURE;
    }
    if((dimension <= 0) || (dimension > 3))
    {
        fprintf(stderr,"error: dimension is not specified or invalid.\n");
        return EXIT_FAILURE;
    }


    convert_phi_to_psi(input_density,output_psi,dimension);


    return EXIT_SUCCESS;
}

/*---------------------------------------------------------------------------*/
int process_option(int argc,char **argv,char **input_density,
                   char **output_psi,int *dimension)
{
    int c;
    int option_index;
    char *program_name;

    program_name = argv[0];

    do
    {
        option_index = 0;
        c = getopt_long(argc,argv,optstring,longopts,&option_index);

        switch(c)
        {
            case 'i':
                    /*  input density file  */
                *input_density = optarg;
                break;

            case 'o':
                    /*  output psi file  */
                *output_psi = optarg;
                break;

            case 'd':
                    /*  dimension  */
                sscanf(optarg,"%d",dimension);
                break;

            case 'v':
                    /*  version  */
                fprintf(stderr,copyingstring,programstring,PACKAGE,VERSION);
                exit(EXIT_SUCCESS);
                break;

            case 'h':
                    /*  help  */
                fprintf(stderr,helpstring,program_name);
                exit(EXIT_SUCCESS);
                break;

            case '?':
                    /*  invalid option  */
                exit(EXIT_FAILURE);
                break;

            case -1:
                break;

            default:
                fprintf(stderr,
                        "error: getopt returned character code 0%o\n", c);
                return 1;
        }
    }
    while(c != -1);


    if((optind < argc) && (*input_density == NULL))
    {
            /*  input density file  */
        *input_density = argv[optind++];
    }
    if((optind < argc) && (*output_psi == NULL))
    {
            /*  output psi file  */
        *output_psi = argv[optind++];
    }

    return 0;
}

/*---------------------------------------------------------------------------*/
void convert_phi_to_psi(char *input_density,char *output_psi,int dimension)
{
    gzFile fp1,fp2;
    char *buf;
    int i,n;
    double u;

    if((input_density == NULL) || (strlen(input_density) == 0)
       || (output_psi == NULL) || (strlen(output_psi) == 0))
        return;

    buf = xmalloc(sizeof(char) * ZGETS_BUFFERSIZE);

    fp1 = gzopen(input_density,"rb");
    fp2 = gzopen(output_psi,"wb");

    while(gzeof(fp1) == 0)
    {
        gzgets(fp1,buf,ZGETS_BUFFERSIZE);

            /*  skip white spaces.  */
        i = 0;
        while(isspace(buf[i]))
            i++;

            /*  skip first elements which correspond to lattice points.  */
        n = 0;
        while((n < dimension) && (buf[i] != '\0'))
        {
            if(isspace(buf[i]))
            {
                n++;
                do
                {
                    i++;
                } while(isspace(buf[i]) && (buf[i] != '\0'));
            }
            else
            {
                i++;
            }
        }

        if(n < dimension)
            continue;

        while(sscanf(buf + i,"%lf",&u) == 1)
        {
            gzprintf(fp2,"%g ",sqrt(u));

                /*  proceed to the next component.  */
            while((!isspace(buf[i])) && (buf[i] !='\0'))
                i++;
            while(isspace(buf[i]) && (buf[i] != '\0'))
                i++;
        }
        gzprintf(fp2,"\n");
    }

    gzclose(fp1);
    gzclose(fp2);

    free(buf);
}

/*---------------------------------------------------------------------------*/
