/*---------------------------------------------------------------------------*/

/*!  \file  textwrite.c
     \brief  output the field data as ascii text format.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005,2006,2018 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#include <stdio.h>
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifdef HAVE_ZLIB_H
#include <zlib.h>
#endif

#include "textwrite.h"

/*---------------------------------------------------------------------------*/
static void write_field_text_with_lattice_1d(char *filename,
                                             field_geometry *geometry,
                                             double **u,int n);
static void write_field_text_with_lattice_2d(char *filename,
                                             field_geometry *geometry,
                                             double **u,int n);
static void write_field_text_with_lattice_3d(char *filename,
                                             field_geometry *geometry,
                                             double **u,int n);

/*---------------------------------------------------------------------------*/
void write_field_text(char *filename,field_geometry *geometry,double **u,int n)
{
    gzFile fp;
    int i,j;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    fp = gzopen(filename,"wb");

    for(i = 0;i < lattice_point_number(geometry);i++)
    {
        for(j = 0;j < n;j++)
        {
            gzprintf(fp,"%g ",u[j][i]);
        }
        gzprintf(fp,"\n");
    }

    gzclose(fp);
}

/*---------------------------------------------------------------------------*/
void write_field_text_with_lattice(char *filename,field_geometry *geometry,
                                   double **u,int n)
{
    if((filename == NULL) || (strlen(filename) == 0))
        return;

    switch(geometry->dimension)
    {
        case 1:
            write_field_text_with_lattice_1d(filename,geometry,u,n);
            break;

        case 2:
            write_field_text_with_lattice_2d(filename,geometry,u,n);
            break;

        case 3:
            write_field_text_with_lattice_3d(filename,geometry,u,n);
            break;

        default:
            break;
    }
}

/*---------------------------------------------------------------------------*/
void write_field_text_with_lattice_1d(char *filename,
                                      field_geometry *geometry,
                                      double **u,int n)
{
    gzFile fp;
    int x,i;
    double dx;

    dx = geometry->l[0] / geometry->n[0];

    fp = gzopen(filename,"wb");

    for(x = 0;x < geometry->n[0];x++)
    {
        gzprintf(fp,"%g ",x * dx);
        for(i = 0;i < n;i++)
        {
            gzprintf(fp,"%g ",u[i][x]);
        }
        gzprintf(fp,"\n");
    }

    gzclose(fp);
}

/*---------------------------------------------------------------------------*/
void write_field_text_with_lattice_2d(char *filename,
                                      field_geometry *geometry,
                                      double **u,int n)
{
    gzFile fp;
    int x,y,i;
    double dx,dy;

    dx = geometry->l[0] / geometry->n[0];
    dy = geometry->l[1] / geometry->n[1];

    fp = gzopen(filename,"wb");

    for(x = 0;x < geometry->n[0];x++)
    {
        for(y = 0;y < geometry->n[1];y++)
        {
            gzprintf(fp,"%g %g ",x * dx,y * dy);
            for(i = 0;i < n;i++)
            {
                gzprintf(fp,"%g ",u[i][x * geometry->n[1] + y]);
            }
            gzprintf(fp,"\n");
        }
        gzprintf(fp,"\n");
    }

    gzclose(fp);
}

/*---------------------------------------------------------------------------*/
void write_field_text_with_lattice_3d(char *filename,
                                      field_geometry *geometry,
                                      double **u,int n)
{
    gzFile fp;
    int x,y,z,i;
    double dx,dy,dz;

    dx = geometry->l[0] / geometry->n[0];
    dy = geometry->l[1] / geometry->n[1];
    dz = geometry->l[2] / geometry->n[2];

    fp = gzopen(filename,"wb");

    for(x = 0;x < geometry->n[0];x++)
    {
        for(y = 0;y < geometry->n[1];y++)
        {
            for(z = 0;z < geometry->n[2];z++)
            {
                gzprintf(fp,"%g %g %g ",x * dx,y * dy,z * dz);
                for(i = 0;i < n;i++)
                {
                    gzprintf(fp,"%g ",u[i][(x * geometry->n[1] + y) * geometry->n[2] + z]);
                }
                gzprintf(fp,"\n");
            }
            gzprintf(fp,"\n");
        }
        gzprintf(fp,"\n");
    }

    gzclose(fp);
}

/*---------------------------------------------------------------------------*/
